# Copyright 2025 Softwell S.r.l.
# Licensed under the Apache License, Version 2.0

"""Server application package - default system endpoints."""

from .server_app import ServerApplication

__all__ = ["ServerApplication"]
