# Copyright 2025 Softwell S.r.l.
# Licensed under the Apache License, Version 2.0

"""Genro API Explorer - Custom API documentation application."""

from .genro_api_app import Application

__all__ = ["Application"]
